; muterf84.asm - a PIC 16F84 program to actuate a relay upon receipt of an
;                IR remote control mute signal for a Sony TV
;
;  9/3/2001 -   Chuck Olson, WB9KZY
;               Jackson Harbor Press
;               http://jacksonharbor.home.att.net/ham.htm
;               jacksonharbor@att.net
;
;
;       What does it do?
;
;       This program uses a 16F84 along with an IR remote decoder module
;       and a relay to turn off a speaker (via the relay) when a Sony
;       IR remote (set for a Sony TV) sends a mute signal.
;
;
;       Possible Enhancements:
;       
;       a) control a motorized pot (Mouser) to add true volume control
;
;       b) General infrared remote control usage for ham related projects
;
;
; 16F84 Pinout:
;
;       pin #   name    connection
;       -----   -----   ----------
;        1      RA2     no connection
;        2      RA3     no connection
;        3      RA4     no connection
;        4      mclr    10k resistor to +5V
;        5      VSS     Ground
;        6      RB0     IR receiver module output
;        7      RB1     no connection
;        8      RB2     no connection
;        9      RB3     to relay 
;       10      RB4     no connection
;       11      RB5     no connection
;       12      RB6     no connection
;       13      RB7     no connection
;       14      VDD     +5V supply
;       15      osc1    4 Mhz crystal/resonator, 22 pf cap to ground
;       16      osc2    4 Mhz crystal/resonator, 22 pf cap to ground
;       17      RA1     no connection
;       18      RA0     no connection
        
        
        
        LIST P=16F84, R=DEC 
        INCLUDE "p16F84.inc"
        
 
 
 __FUSES _PWRTE_OFF & _CP_OFF & _WDT_OFF & _XT_OSC 
; powerup timer off - still not sure why
; copy protect is off
; watchdog timer is off 
; the oscillator uses an external 4 Mhx crystal or resonator



; memory locations  -   0C to 4F (hex) available in 16F84
;                       20 to 7F (hex) in 16F628, so let's start with 20
;       Note that the CBLOCK/ENDC assembler directives make it easy to add or
;               remove memory locations.
        CBLOCK  0x20
        ircode:         1       ;holds "built-up" remote operation code
        bitcntr:        1       ;bit counter for IR routines
        plenh:          1       ;the high pulse length counter
        plenl:          1       ;the low pulse length counter
        upprcnt:        1       ;the upper (x 12 ms) counter
        temp:           1       ;a temporary byte for the 50 us delay routine
                ENDC
        


; bit usage
IR      equ     0               ;bit 0 of portb is the infrared decoder pin
relay   equ     3               ;bit 3 of portb is the relay output pin
carry   equ     0               ;carry flag bit of STATUS        
zero    equ     2               ;zero flag bit of STATUS
w       equ     0               ;destination
f       equ     1               ;           registers



        ORG     0x00
start   goto    init    ;startup routine
        


        ORG     0x04
rupt    goto    init    ;interrupt routine



; del50us - a routine to wait 50 us
; total delay = 2 (call) + 1 (movlw) + 1 (movwf) + 3 (x) - 1 (last (0) loop) 
;               + 5 (nops & return) = 50 usec
;               8 + 3x = 50   =>   3x = 42   =>   x = 14
del50us movlw   d'14'           ;1 load up a .1 ms count into x reg
        movwf   temp            ;1 save it temporarily
d50usl  decfsz  temp,f          ;1 (2) decrement the x reg
        goto    d50usl          ;2 test to see if x reg is now 0, loop if not
        nop                     ;1 waste a cycle
        nop                     ;1 waste a cycle
        nop                     ;1 waste a cycle
        return                  ;2 end of 50 us delay routine



; init - the initialization power up entry point of the program
init    clrf    PORTA           ;clear all bits of port a
        clrf    PORTB           ;clear all bits of port b, too
        bsf     STATUS,5        ;set the bank bit to 1
        movlw   b'00000001'     ;all outs on PORTB except bit 0 (IR module)
        movwf   TRISB           ;set the PORT B tri-state register
        movlw   b'00000000'     ;all outputs on PORTA
        movwf   TRISA           ;set the PORT A tri-state register
        movlw   b'00000000'     ;no option bits are relevant except PORTB p/u on
        movwf   OPTION_REG      ;set the option register
        bcf     STATUS,5        ;set the bank bit to 0
        bcf     INTCON,GIE      ;disable interrupts - needed ?
        bsf     INTCON,INTE     ;enable the port b, bit 0 interrupt
        ; Why disable the global interrupts, but enable the RB0 interrupt?
        ;       For the sleep command below - the sleep mechanism requires
        ;       that the INTE bit be enabled, but doesn't require that 
        ;       interrupts (GIE bit) are actually used.



; mainlup - the main program loop starts here        
mainlup ;the next two statements allow the PIC to sleep (stop the clock)
;       until there is some input from the IR decoder module.  While this
;       does reduce current usage somewhat, the real reason for putting the
;       PIC to sleep here is that stopping the PIC clock
;       eliminates a source of possible EMI.  
        bcf     INTCON,INTF     ;clear the port b, bit 0 interrupt flag
        sleep                   ;wait for a high to low transition on RB0
;        
; an IR remote button has been pressed
pream   btfss   PORTB,IR        ;is the ir decoder still outputting a 0 ?
        goto    pream           ;yes, keep looping until the preamble is over
        clrf    ircode          ;clear the ir code storage byte
        clrf    bitcntr         ;clear the bit counter for the ir code storage byte
;
; now get the data
phi     clrf    plenh           ;clear the high pulse length counter
incplen incf    plenh,f         ;increment the counter
        call    del50us         ;wait a twentieth of a ms
;
;test the high pulse to make sure the pulse isn't too long
        movf    plenh,w         ;load up 12 ms (240 x .05 ms) for comparison
        sublw   d'240'          ;is plenh greater than 12 ms?
        btfss   STATUS,carry    ;carry will be 0 if plenh > 240
        goto    cmdun           ;yes, it's > 240, bail out
;
;no, test the input to see if it's still high
        btfsc   PORTB,IR        ;is the ir decoder still outputting a 1 ?
        goto    incplen         ;yes, loop back if still high
        clrf    plenl           ;clear the low pulse length counter
incplel incf    plenl,f         ;increment the counter
        call    del50us         ;wait a 20th of a ms
        btfss   PORTB,IR        ;is the ir decoder still outputting a 0 ?
        goto    incplel         ;yes, loop back and increment the counter
        ;the low pulse is over
        incf    bitcntr,f       ;increment the ir code byte bit counter
        movf    bitcntr,w       ;load the bit counter for comparison
        sublw   d'8'            ;test the bit count against 8
        btfss   STATUS,carry    ;is the bit count > 8 ?
        goto    cmdun           ;yes, bail out - only first 8 bits are used
;
; only the low count is examined 
;       if it's less than 18 then it's a zero 
;       if it's greater than 18 it's a one.
        movf    plenl,w         ;load up the low count
        sublw   d'18'           ;compare it to 18
        btfsc   STATUS,carry    ;carry will be 0 if plenl > 18
        goto    azero           ;plenl < 18, so the bit is a zero
        ;plenl > 18 so the bit is a one
aone    bsf     STATUS,carry    ;set the carry bit to allow shifting in a one
        rlf     ircode,f        ;shift in the one into ir code to build the data word
        goto    phi             ;continue the data gathering
azero   ;else, you have a zero
        bcf     STATUS,carry    ;clear the carry bit
        rlf     ircode,f        ;shift a zero into ir code to build the data word
        goto    phi             ;continue the data gathering
cmdun   movf    ircode,w        ;load up the resulting first 8 bits of the keypress
        sublw   0x29            ;compare it to a 29 (hex) - mute
        btfsc   STATUS,zero     ;is it a mute?
        goto    onesec          ;yes, toggle the output
        goto    loopfin         ;no, skip the relay touches and finish the loop
onesec  btfsc   PORTB,relay     ;is the relay on (pin hi) now?
        goto    rledoff         ;yes, go and turn it off
;
;no, turn it on
rledon  bsf     PORTB,relay     ;turn on the relay
        goto    loopfin         ;finish the loop
rledoff ;yes, turn it off
        bcf     PORTB,relay     ;turn off the relay
loopfin clrf    plenh           ;clear the high pulse length counter
        clrf    upprcnt         ;clear the upper counter, too!
loopfp1 btfss   PORTB,IR        ;loop here until IR pulse goes high
        goto    loopfp1         ;IR is still low, loop back
incpl2  incf    plenh,f         ;increment the counter
        call    del50us         ;wait .05 ms
        movf    plenh,w         ;load up 12 ms (240 x .05 ms) for comparison
        sublw   d'240'          ;compare plenh to 240 (12 ms)
        btfss   STATUS,carry    ;is plenh greater than 12 ms?
        goto    incupc          ;yes, increment the upper counter
;
;no, get test the input to see if it's still high
tstir   btfsc   PORTB,IR        ;is the IR input high ?
        goto    incpl2          ;yes, loop back and increment the counter
        goto    loopfin         ;the IR input is low, wait until it goes high again
incupc  ;increment the upper counter and see if the command stream is over
; the command stream is over if the total high time is 84 ms or more (count=7)
        incf    upprcnt,f       ;increment the upper counter
        clrf    plenh           ;clear the pulse high counter
        movf    upprcnt,w       ;load up 12 ms (240 x .05 ms) for comparison
        sublw   d'7'            ;compare to 7 upper counts (84 ms)
        btfss   STATUS,carry    ;is the upper count > 7 ?    
        goto    mainlup         ;yes, go back to the main loop
        goto    tstir           ;no, return to the ir high? test loop


        end
